# -*- coding: utf-8 -*-
"""
Created on Thu Jan  6 23:35:23 2022

@author: Avram Gottschlich
"""
"""
I copied several functions from your code;
if it's easier to refer to them rather than copying them that's absolutely fine

This rewrites the html each time it is called
If it's easy to call the puzzle/round functions each time they're updated,
that would be great

Requires sorttable.js, which should be included
"""
from turbot.channel import channel_url
from boto3.dynamodb.conditions import Key

website = "https://halibut.cworth.org/"
#change this if we're using AWS or some other subdomain instead

def find_hunt_for_hunt_id(turb, hunt_id):
    """Given a hunt ID find the database item for that hunt

    Returns None if hunt ID is not found, otherwise a
    dictionary with all fields from the hunt's row in the table,
    (channel_id, active, hunt_id, name, url, sheet_url, etc.).
    """
    response = turb.table.get_item(
        Key={
            'hunt_id': hunt_id,
            'SK': 'hunt-{}'.format(hunt_id)
            })

    if 'Item' in response:
        return response['Item']
    else:
        return None

def hunt_puzzles_for_hunt_id(turb, hunt_id):
    """Return all puzzles that belong to the given hunt_id"""

    response = turb.table.query(
        KeyConditionExpression=(
            Key('hunt_id').eq(hunt_id) &
            Key('SK').begins_with('puzzle-')
            )
        )

    return response['Items']

def elink(lin, text):
    #shortcutting function for creating html links
    #opens in a new tab for external links
    return '<a href="{}" target="_blank" rel="noreferrer noopener">{}</a>'.format(lin, text)

def link(lin, text):
    #internal links, doesn't open new tab
    return '<a href="{}">{}</a>'.format(lin, text)

def hunt_info(turb, hunt):
    """
    Retrieves list of rounds, puzzles for the given hunt
    """
    name = hunt["name"]
    hunt_id = hunt["hunt_id"]
    channel_id = hunt["channel_id"]

    puzzles = hunt_puzzles_for_hunt_id(turb, hunt_id)

    rounds = set()
    for puzzle in puzzles:
        if "rounds" not in puzzle:
            continue
        for rnd in puzzle["rounds"]:
            rounds.add(rnd)
    rounds = list(rounds)
    rounds.sort()

    return puzzles, rounds

def round_stat(rnd, puzzles):
    #Counts puzzles, solved, list of puzzles for a given round
    puzzle_count = 0
    solved_count = 0
    solved_puzzles = []
    unsolved_puzzles = []
    metas = []
    meta_solved = 0
    for puzzle in puzzles:
        if "rounds" not in puzzle:
            continue
        if rnd in puzzle["rounds"]:
            if puzzle['type'] == 'meta':
                metas.append(puzzle)
                if puzzle['status'] == 'solved':
                    meta_solved += 1
            else:
                puzzle_count += 1
                if puzzle['status'] == 'solved':
                    solved_count += 1
                    solved_puzzles.append(puzzle)
                else:
                    unsolved_puzzles.append(puzzle)
    solved_puzzles = sorted(solved_puzzles, key = lambda i: i['name'])
    unsolved_puzzles = sorted(unsolved_puzzles, key = lambda i: i['name'])
    rnd_puzzles = metas + unsolved_puzzles + solved_puzzles
    return puzzle_count, solved_count, rnd_puzzles, meta_solved, len(metas)



def overview(puzzles, rounds):
    #big board, main page. saves as index.html
    start = ['<!DOCTYPE html>\n',
     '<html>\n',
     '<head>\n',
     '  <meta charset="utf-8">\n',
     '  <meta name="viewport" content="width=device-width, initial-scale=1">\n',
     '\n',
     '  <link rel="stylesheet" href="overview.css">\n',
     '  <script type="text/javascript">\n',
     '    // Hide all elements with class="containerTab", except for the one that matches the clickable grid column\n',
     '    function openTab(tabName) {\n',
     '      var i, x;\n',
     '      x = document.getElementsByClassName("containerTab");\n',
     '      for (i = 0; i < x.length; i++) {\n',
     '        x[i].style.display = "none";\n',
     '      }\n',
     '      document.getElementById(tabName).style.display = "block";\n',
     '    }\n',
     '  </script>\n',
     '\n',
     '  <title>Hunt Overview</title>\n',
     '  <script src="sorttable.js"></script>\n'
     '</head>\n',
     '    <div class="sidenav">\n'
     '      <a href="index.html">Hunt Overview</a>'
     '      <a href="all.html">All Puzzles</a>\n'
     '      <a href="unsolved.html">Unsolved</a>\n'
     '      <a href="solved.html">Solved</a>\n'
     '    </div>\n'
     '<body>\n',]
    columns = ['  <div class="row">\n']
    expanding = []
    i = 1
    for rnd in rounds:
        puzzle_count, solved_count, rnd_puzzles, meta_solved, metas = round_stat(rnd, puzzles)
        if metas == meta_solved and metas > 0:
            status = 'solved'
        else:
            status = 'unsolved'
        columns += ['    <div class="column {}" onclick="openTab(\'b{}\');">\n'.format(status, i),
        '      <p>{}</p>\n'.format(rnd),
        '      <p>Puzzles: {}/{}</p>\n'.format(solved_count, puzzle_count),
        '      <p>Metas: {}/{}</p>\n'.format(meta_solved, metas),
        '    </div>\n']

        expanding += [
        '  <div id="b{}" class="containerTab {}" style="display:none;">\n'.format(i, status),
        '    <span onclick="this.parentElement.style.display=\'none\'" class="closebtn">x</span>\n',
        '    <h2>{}</h2>\n'.format(link(website + "_".join(rnd.split()) + "_round.html", rnd)),
        '    <table class="sortable">\n',
        '      <tr>\n',
        '        <th><u>Puzzle</u></th>\n',
        '        <th><u>Answer</u></th>\n',
        '      </tr>\n',]

        for puzzle in rnd_puzzles:
            if puzzle['type'] == 'meta':
                meta = ' [META]'
            else:
                meta = ''
            if puzzle['status'] == 'solved':
                expanding += ['      <tr class=\'solved\';>\n',
                '        <td>{}</td>\n'.format(link(website + "_".join(puzzle['name'].split()) + ".html", puzzle['name']+meta)),
                '        <td>{}</td>\n'.format(puzzle['solution']),
                '      </tr>\n']
            else:
                expanding += ['      <tr class=\'unsolved\';>\n',
                '        <td><b>{}</b></td>\n'.format(link(website + "_".join(puzzle['name'].split()) + ".html", puzzle['name']+meta)),
                '        <td></td>\n',
                '      </tr>\n']
        expanding.append('    </table>\n')
        expanding.append('  </div>\n')
        i += 1
    columns.append('  </div>\n')
    end = ['</body>\n', '</html>\n']
    html = start + expanding + columns + end
    file = "index.html"
    f = open(file, "w")
    for line in html:
        f.write(line)
    f.close()
    return None

def round_overview(rnd, puzzles):
    #inputs: round name, puzzles
    #round overview page
    #saves as (round name)_round.html, in case meta/round share names.
    #underscores replace spaces for links.
    rnd_puzzles, meta_solved, metas = round_stat(rnd, puzzles)[2:]
    if meta_solved == metas and metas > 0:
        status = 'solved'
    else:
        status = 'unsolved'
    start = ['<html>\n',
     '    <head>\n',
     '        <link rel="stylesheet" href="individual.css">\n',
     '        <title>Mystery Hunt 2022</title>\n',
     '        <script src="sorttable.js"></script>\n',
     '    </head>\n',
     '    <body class="{}">\n'.format(status),
     '        <h1><b>{}</b></h1>\n'.format(rnd),
     '        <p>{}</p>\n'.format(link(website + "index.html", 'Hunt Overview')),
     '        <div>\n',
     '            <table class="center sortable">\n',
     '                <thead>\n',
     '                    <tr>\n',
     '                        <th>Puzzle Title/Slack</th>\n',
     '                        <th>Puzzle Link</th>\n',
     '                        <th>Sheet</th>\n',
     '                        <th>Overview</th>\n',
     '                        <th>Answer</th>\n',
     #'                        <th>Extra Links</th>\n',
     '                        <th>Tags</th>\n',
     '                    </tr>\n',
     '                </thead>\n',
     '                <tbody>\n']
    puzzle_list = []
    for puzzle in rnd_puzzles:
        if puzzle['type'] == 'meta':
            meta = ' [META]'
        else:
            meta = ''
        slack_url = channel_url(puzzle['channel_id'])

        if puzzle['status'] == 'solved':
            puzzle_list += [ '                    <tr>\n',
             '                        <td>{}</td>\n'.format(elink(slack_url, puzzle['name']+meta)),
             '                        <td>{}</td>\n'.format(elink(puzzle['url'], 'Puzzle')),
             '                        <td>{}</td>\n'.format(elink(puzzle['sheet_url'], 'Sheet')),
             '                        <td>{}</td>\n'.format(link(website + "_".join(puzzle['name'].split()) + '.html', 'Overview')),
             '                        <td>{}</td>\n'.format(puzzle['solution']),
            # '                        <td></td>\n',
             '                        <td>{}</td>\n'.format("".join(puzzle['tags'])),
             '                    </tr>\n']
        else:
            puzzle_list += [ '                    <tr>\n',
             '                        <td><b>{}</b></td>\n'.format(elink(slack_url, puzzle['name']+meta)),
             '                        <td>{}</td>\n'.format(elink(puzzle['url'], 'Puzzle')),
             '                        <td>{}</td>\n'.format(elink(puzzle['sheet_url'], 'Sheet')),
             '                        <td>{}</td>\n'.format(link(website + "_".join(puzzle['name'].split()) + '.html', 'Overview')),
             '                        <td></td>\n',
            # '                        <td></td>\n',
             '                        <td>{}</td>\n'.format(" ".join(puzzle['tags'])),
             '                    </tr>\n']
    end = ['                </tbody>\n',
    '            </table>\n',
    '        </div>\n',
    '    </body>\n',
    '</html>\n']
    html = start + puzzle_list + end
    file = "{}_round.html".format('_'.join(rnd.split()))
    f = open(file, "w")
    for line in html:
        f.write(line)
    f.close()
    return None

def puzzle_overview(puzzle):
    #overview page for individual puzzles. saves as (name of puzzle).html,
    #with underscores rather than spaces
    name = puzzle['name']
    if puzzle['type'] == 'meta':
        meta = ' [META]'
    else:
        meta = ''
    slack_url = channel_url(puzzle['channel_id'])
    round_url = [link(website + "_".join(rnd.split()) + "_round.html", rnd) for rnd in puzzle['rounds']]
    if puzzle['status'] == 'solved':
        solution = puzzle['solution']
        status = 'solved'
    else:
        solution = ""
        status = 'unsolved'
    html = ['<!DOCTYPE html>\n',
     '<html>\n',
     '<head>\n',
     '    <meta charset="utf-8">\n',
     '    <meta name="viewport" content="width=device-width, initial-scale=1">\n',
     '    <link rel="stylesheet" href="individual.css">\n',
     '    <title>{}</title>\n'.format(name+meta),
     '    <p>{}</p>'.format(link(website + 'index.html', 'Hunt Overview')),
     '\n',
     '</head>\n',
     '<body class="{}">\n'.format(status),
     '    <h1>{}</h1>\n'.format(name+meta),
     '    <div>\n',
     '        <table class="center">\n',
     '            <tr>\n',
     '                <td>{}</td>\n'.format(elink(slack_url, 'Channel')), #slack channel
     '                <td>{}</td>\n'.format(elink(puzzle['url'], 'Puzzle')),
     '                <td>{}</td>\n'.format(elink(puzzle['sheet_url'], 'Sheet')),
     '                <td>Additional Resources</td>\n',
     '            </tr>\n',
     '        </table>\n',
     '        <table class="center">\n',
     '            <tr>\n',
     '                <td>Round(s): {}</td>\n'.format(" ".join(round_url)), #round page on our site
     '                <td>Tags: {}</td>\n'.format(" ".join(puzzle['tags'])), #add tags
     '            </tr>\n',
     '            <tr>\n',
     '                <td>Answer: {}</td>\n'.format(solution),
     '                <td>State: {}</td>\n'.format(puzzle['status']),
     '            </tr>\n',
     '        </table>\n',
     '    </div>\n',
     '\n',
     '</body>\n',
     '</html>\n']
    underscored = "_".join(name.split())
    file = "{}.html".format(underscored)
    f = open(file, "w")
    for line in html:
        f.write(line)
    return None

def puzzle_lists(puzzles, filt):
    #filt is one of "All", "Solved", "Unsolved" and spits out the appropriate list of puzzles
    #generates pages for all puzzles, solved puzzles, unsolved puzzles
    #saves as all/solved/unsolved.html, has a sidebar to link to each other and to the big board
    solved_puzzles = [puzzle for puzzle in puzzles if puzzle['status'] == 'solved']
    unsolved_puzzles = [puzzle for puzzle in puzzles if puzzle['status'] != 'solved']
    start = ['<html>\n',
     '    <head>\n',
     '        <link rel="stylesheet" href="overview.css">\n',
     '        <title>Mystery Hunt 2022</title>\n',
     '        <script src="sorttable.js"></script>\n',
     '    </head>\n',
     '    <div class="sidenav">\n'
     '      <a href="index.html">Hunt Overview</a>'
     '      <a href="all.html">All Puzzles</a>\n'
     '      <a href="unsolved.html">Unsolved</a>\n'
     '      <a href="solved.html">Solved</a>\n'
     '    </div>\n'
     '    <body>\n',
     '        <h1><b>{}</b></h1>\n'.format('{} Puzzles').format(filt),
     '        <p>{}</p>\n'.format(link(website + "index.html", 'Hunt Overview')),
     '        <div>\n',
     '            <table class="center sortable">\n',
     '                <thead>\n',
     '                    <tr>\n',
     '                        <th>Puzzle Title/Slack</th>\n',
     '                        <th>Puzzle Link</th>\n',
     '                        <th>Sheet</th>\n',
     '                        <th>Overview</th>\n',
     '                        <th>Answer</th>\n',
     '                        <th>Round(s)</th>\n',
     '                        <th>Tags</th>\n',
     '                    </tr>\n',
     '                </thead>\n',
     '                <tbody>\n']
    solved_code = []
    unsolved_code = []
    for puzzle in solved_puzzles:
        if puzzle['type'] == 'meta':
            meta = ' [META]'
        else:
            meta = ''
        slack_url = channel_url(puzzle['channel_id'])
        round_url = link(website + "_".join(rnd.split()) + "_round.html", puzzle['rounds'][0])
        #assuming one round per puzzle for now

        solved_code += ['                    <tr>\n',
         '                        <td>{}</td>\n'.format(elink(slack_url, puzzle['name']+meta)),
         '                        <td>{}</td>\n'.format(elink(puzzle['url'], 'Puzzle')),
         '                        <td>{}</td>\n'.format(elink(puzzle['sheet_url'], 'Sheet')),
         '                        <td>{}</td>\n'.format(link(website + "_".join(puzzle['name'].split()) + '.html', 'Overview')),
         '                        <td>{}</td>\n'.format(puzzle['solution']),
         '                        <td>{}</td>\n'.format(round_url),
         '                        <td>{}</td>\n'.format("".join(puzzle['tags'])),
         '                    </tr>\n']
    for puzzle in unsolved_puzzles:
        if puzzle['type'] == 'meta':
            meta = ' [META]'
        else:
            meta = ''
        slack_url = channel_url(puzzle['channel_id'])
        round_url = link(website + "_".join(rnd.split()) + "_round.html", puzzle['rounds'][0])
        #assuming one round per puzzle for now

        unsolved_code += ['                    <tr>\n',
         '                        <td>{}</td>\n'.format(elink(slack_url, puzzle['name']+meta)),
         '                        <td>{}</td>\n'.format(elink(puzzle['url'], 'Puzzle')),
         '                        <td>{}</td>\n'.format(elink(puzzle['sheet_url'], 'Sheet')),
         '                        <td>{}</td>\n'.format(link(website + "_".join(puzzle['name'].split()) + '.html', 'Overview')),
         '                        <td></td>\n',
         '                        <td>{}</td>\n'.format(round_url),
         '                        <td>{}</td>\n'.format("".join(puzzle['tags'])),
         '                    </tr>\n']
    end = ['                </tbody>\n',
    '            </table>\n',
    '        </div>\n',
    '    </body>\n',
    '</html>\n']
    if filt == "All":
        file1 = 'all.html'
        f = open(file1, "w")
        for line in start + unsolved_code + solved_code + end:
            f.write(line)
        f.close()
    elif filt == "Solved":
        file2 = 'solved.html'
        f = open(file2, 'w')
        for line in start + solved_code + end:
            f.write(line)
        f.close()
    elif filt == "Unsolved":
        file3 = 'unsolved.html'
        f = open(file3, 'w')
        for line in start + unsolved_code + end:
            f.write(line)
        f.close()
    return None



puzzles, rounds = hunt_info(turb, hunt)
#I am not sure where these come from
overview(puzzles, rounds)
for rnd in rounds:
    round_overview(rnd, puzzles)
for puzzle in puzzles:
    puzzle_overview(puzzle)
puzzle_lists(puzzles, "All")
puzzle_lists(puzzles, "Solved")
puzzle_lists(puzzles, "Unsolved")
